from distutils.core import setup

setup(
    name='yi-mp',
    version='1.0.5',
    py_modules=['emp_wifi', 'emp_boot', 'ide', 'emp_utils', 'emp_webrepl'],
    author='1045149087@qq.com',
    author_email='1045149087@qq.com',
    url='https://github.com/HWYWL/yi-mp',
    description='(YI MicroPython) is a upy module to make things Easy on MicroPython.'
)
